/*
Licensed Materials - Property of IBM

 Copyright IBM Corp. 2012 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or
disclosure restricted by GSA ADP Schedule Contract with
IBM Corp.
*/ 
define( [ "dojo/_base/declare",
          "dijit/_WidgetBase",
          "dojo/dom-construct",
          "dojo/dom-geometry",
          "dojo/dom-style",
          "dojo/dom-class",
          "dojo/dom-attr",
          "dojo/query",
          "dojo/on",
          "dojo/_base/lang",
          "dijit/popup",
          "dijit/registry",
          "dojo/keys",
          "dojo/_base/event",
          "dojo/_base/window",
          "dojo/dom-geometry",
          "dojox/image",
					"dijit/Dialog",
					"dijit/form/Button" ],
          function( declare, _WidgetBase, domConst, domGeo, domStyle, domClass, domAttr, $, on, lang, popup, registry, keys, Event, win, domGeo, image, Dialog, Button ){
            
   return declare( "ModelControl", _WidgetBase, {
     /* This class makes some assumptions about how the class attribute of the DOM elements are assigned
        The input is a list of names like: [ file, files, import, ... ]
        
        The code then assumes that the main DOM node target is <keyPrefix><inputlist[i]>.  That node will get
        the mouse event.  In addition it will get a new data-key attribute that will be used by the 
        hover method to determine the next step.
        
        The hover method assume the highlite CSS classes have the data-key name appended.  In addition, 
        it assumes that the id of the flyout node is flyout-<data-key>. 
        
        For example, say the key is import.  The code expects to find the following:
        1. DOM node with a class attribute of sda-import
        2. A highlite class of import (used in the CSS as .import .sda-import for example).
        3. A flyout with an id of flyout-import.
        
        When the assumptions are not meet, the code ignores the issues.  No flyout no big deal.
      */
     
     debug: false,
     
     // List of ids in the dom
     keys: null,
     
     // Current CSS class used for the highlite
     activeKey: null,
     
     // Prefix key that the model is using for example, sda- or mda-
     keyPrefix: "",
     
     // Based on the key name determines the orientation of a tool tip
     //tooltipOrient: [],
     
     // Event Registry to prevent adding to many handlers to the flyout.
     eventReg: [],
	 
	 
	 // list of lists of mutually exclusive groups for on_mouse_enter  on_mouse_leave actions
     mutuallyExclusiveGroups: [],
     
     // A list of flyouts to monitor with oritentations each index has
     // { flyout: _WidgetBase, orient: [''] }
     // For faster indexing this acts as a hashtable where the flyout id
     // will locate the above object. 
     flyouts: [],
     
		 // Indicates if we are printing based on the URL having ?print
		 printing: false,
		 
		 // List of actions to perform
		 actionList: null,
		 
     constructor: function( keys, keyPrefix ) {
        this.keys = keys;
        this.keyPrefix = keyPrefix;

        if( window.location.search.indexOf( "print" ) !== -1 ){
				   this.printing = true;
				}
     },
     
     /* postCreate
      * 
      * Searchs for the DOM nodes in the model listed in keys and connects a 
      * mouse event handler.  When printing is enabled, the events do not 
			* get connect thus creating a static model.
      *
      * Input:
      *  None.
      * 
      * Output:
      * None.
      *
      * Side Effects:
      *   DOM nodes given as keys will get a data-key attribute assigned.
      *
      */    
     postCreate: function(){
       var hoverFunc = lang.hitch( this, this.onHover ), i,
           keyFunc = lang.hitch( this, this.onKeyPress ), model = $( ".model" )[0];
         
				 /* on( model, 'mouseover', hoverFunc );
				 on( model, 'keypress', keyFunc ); */
				  
				 // Connect up the hover events for all the list elements
				 for( i = 0; i < this.keys.length; ++i ){
					 var div = $("." + this.keyPrefix + this.keys[i])[0] || null;
					 
					 if( div !== null ){
						 // Don't connect the events if the intent is that we're printing
						if( !this.printing ){
							 on( div, 'mouseover', hoverFunc );
							 on( div, 'keypress', keyFunc );
					 	 } 
						 domAttr.set( div, "data-key", this.keys[i] ); 
					 } else if(this.debug) {
						 console.log( "[ModelControl::postCreate] Unable to locate DOM element with classname of " + this.keyPrefix + this.keys[i] );
					 }
				 }// for(...)
     },
     
		/**
     * startup
     *
     * ======  Someone need to call this method =====
		 *
		 * This method will remove the loading screen and determine perform the conversion 
		 * for printing the model.  This method should be called once all setup tasks
		 * for the model are complete. 
     * 
     * Input:
     *  None.
     *
     * Outputs:
     *  None.
     *
     * Side Effects:
     *  Loading screen blocker is hidden. DOM may adjust for printing.
     *
     */   
		 startup: function(){
		    var loading = $(".loading")[0] || null;
				
		 		// Hide the progress indicator  
				if( loading !== null ){                                                                                  
				  domClass.add( loading, "hide" );
				}
				
		    // Are we printing?
		    if( this.printing ){
				  this.printModel();
				} else {
					// Nope, Set focus on the model if possible
					$(".model")[0].focus();
				}
		 },
		 
	/**
     * addMEG  
     *
     * This will add mutually exclusive groups of keys..
		 *
		 * Expect the following JSON:
		 *  [ { name: "<model node key>",
		 *      keys: "<URL to load when the model node is clicked>" } ]
     * 
     * Input:
     *  list - Array of objects in the above format.
     *
     * Outputs:
     *  None.
     *
     */   
		addMEG: function( list ){
			//var i;
			
			//alert(this.mutuallyExclusiveGroups.length);
			
			this.mutuallyExclusiveGroups[this.mutuallyExclusiveGroups.length] = list;
			
		},	 
		 
		 
	/**
     * Preload images  
     *
     * This will add mutually exclusive groups of keys..
		 *
		 * Expect the following JSON:
		 *  [ { name: "<model node key>",
		 *      keys: "<URL to load when the model node is clicked>" } ]
     * 
     * Input:
     *  list - Array of objects in the above format.
     *
     * Outputs:
     *  None.
     *
     */   
		preloadImages: function( list ){
			//console.log( "Preload: " + list );
			dojox.image.preload(list);
			
		},	 
		 
		 
		    
		/**
     * addActions
     *
     * This will add click events to the given keys to perform some action.
		 *
		 * Expect the following JSON:
		 *  [ { key: "<model node key>",
		 *      url: "<URL to load when the model node is clicked>" } ]
     * 
     * Input:
     *  list - Array of objects in the above format.
     *
     * Outputs:
     *  None.
     *
     * Side Effects:
     *  Listeners are added to the DOM node.
     *
     */   
		addActions: function( list ){
			var i, onClickFunc = lang.hitch( this, this.onClick ), div;
			
			if( this.keyPrefix === "" ){
			  console.warn( "[addActions] No key prefix is set, unable to add actions." );
				return;
			}
			
			for( i=0; i < list.length; i+=1){

				// Check the key and url are not blank, no need to add back actions.
				if( list[i].key !== "" || list[i].url !== "" ){
					div = $("." + this.keyPrefix + list[i].key)[0] || null;
					
					// Did we find the node we're looking for?
					if( div !== null ){
						on( div, "click", onClickFunc );
					} else if(this.debug){
						console.log( "[addActions] Unable to locate DOM node for key " + list[i].key + ", skipping action" );
					}
					
				} else if(this.debug) {
					console.log( "[addActions] Detected a blank parameter for the following entry in the array, skipping. " + list[i] );
				}
			} // for
			
			this.actionList = list;
		},
		
		/**
     * onClick
     *
     * Handles the click event generated by the model node.  This will
		 * casue the browser to move to the url given in the action list. 
     * 
     * Input:
     *  evt - DOM event object.
     *
     * Outputs:
     *  None.
     *
     * Side Effects:
     *  Listeners are added to the DOM node.
     *
     */   
		onClick: function(evt){
      var key = this._getKey( evt ), url, i=0;

console.log("click:"+ key);	

      if( key === null ){
				console.warn("[onClick] Unable to determine the key." );
				return;
			}
			
		
			
			// Locate the url, assume previous list check in addList will protect from
			// from bad URLs and bad requests since this method should only be called from
			// click handlers in the node list.  If this assumption breaks, this code breaks.
			while( key !== this.actionList[i].key && i < this.actionList.length ){ i+=1; } 
			url = this.actionList[i].url;
			
			// Adjust the URL for Firefox to remove the extra ../		 
			if( dojo.isFF && url.indexOf( "com.ibm" ) > -1 ){
					url = url.substring( 3 );
			}
			if( window.top !== window ){
			   window.parent.location.assign( url );
			} else {
				window.location.assign( url );
			}
			
			Event.stop( evt );
		},
		
    /**
     * onHover
     *
     * Handle the hover events over the model item.
     * This will add the data-key for the event target node into the 
     * model node class attribute allowing for the model to adjust as necessary.
     * When a flyout is available, this method will locate the flyout and open
     * it.  
     * 
     * Input:
     *  evt - Dojo normalized Mouse event.
     *
     * Outputs:
     *  None.
     *
     * Side Effects:
     * This will stop the event.
     *
     */   
     onHover: function( evt ){
       var model = $(".model")[0] || null, flyout, orient = null , fObj, i;
       var key = this._getKey( evt );
       
       if( model === null ){
         if(this.debug){console.log( "[ModelControl::onHover] Unable to locate the model, bail." );}
         return;
       }
       
       // Ignore duplicate request, which might happen when we give the active now focus 
       if( this.activeKey ==- key ){
         return;
       }
       
       
       /* if the key doesn't match the current flyout,  
       		close the flyout, and remove the activekey
       
       */
       
       fObj = this.flyouts[ "flyout-" + this.activeKey ] || null;
       
       
       	
  		if( fObj === null ){
           if(this.debug){console.log( "[ModelControl::onMouseLeave] Unable to locate the flyout in the object trying the registry. " );}
           flyout = registry.byId( "flyout-" + this.activeKey ) || null;
         } else {
           flyout = fObj.flyout;
         }

 		if( flyout !== null ){
           // For any stack containers, reset them to the first child
           if( flyout.content.toString().indexOf( "StackContainer" ) !== -1 ){
              flyout.content.selectChild( flyout.content.getChildren()[0] );
           }
       		 popup.close( flyout );
         }
       
       
       
       // Remove any previous highlite
       if( this.activeKey !== null ){
         domClass.remove( model, this.activeKey );
         this.activeKey = null;
       }
    

			for( i=0; i < this.mutuallyExclusiveGroups.length; i+=1){
				var x;
				var megi = this.mutuallyExclusiveGroups[i];
				if(megi != null){
					for( x=0; x< megi.idz.length; x+=1){
						var idx = megi.idz[x];
						//if the key is in a mutually exclusive group.. remove the others of that group from the model.
						if(key.indexOf("_box")  !== -1){
							//console.log("test"+key);
							key = key.substring(0, key.indexOf("_box"));
							//console.log(key);
						}
						//console.log("[onHover] key = "+key+" idx = "+idx);
						if(key == idx){
							//if(true){console.log("[onHover] key = idx = "+idx);}
							var y;
							for ( y=0; y< megi.idz.length; y+=1){
								
								var idy = megi.idz[y];
								//if(true){console.log("[onHover] remove = "+idy +"_ome");}
								domClass.remove( model, idy+"_ome");
							}
						}
					}
				}
			} // for
	   
 
       // Add the new highlite to the model
       if( key !== null ){
         domClass.add( model, key );
		 
		 domClass.add( model, key + "_ome"); //ome = on mouse enter.  		  
		 
         this.activeKey = key;
       }
       
       // Now make the flyout appear, if one exists...
       fObj = this.flyouts[ "flyout-" + key ] || null;
       
       if( fObj === null ){
         if(this.debug){console.log( "[ModelControl::onHover] Flyout is not found, looking at registry. " );}
         flyout = registry.byId( "flyout-" + key ) || null;
       }else{
         flyout = fObj.flyout;
         orient = fObj.orient;
       }
       
       if( flyout !== null ){
         if( orient !== null ){
           popup.open( { popup: flyout, around: evt.currentTarget, orient: orient } );
         } else {
          popup.open( { popup: flyout , around: evt.currentTarget } );
         }
         
        
				 // Call startup if a step card is present
				 
				 
         // Do we need to attach an event handler?
         if( typeof( this.eventReg[ key ] ) === "undefined" ){
           this.eventReg[ key ] = flyout.on( "mouseLeave", lang.hitch( this, this.onMouseLeave ) );
           this.eventReg[ key + "_keys" ] = flyout.on( "keypress", lang.hitch( flyout, flyout._onKey ) );
         }
         
         flyout.focus();
				 
         // Apparently onCancel does nothting for a tooltip dialog since the method it inherits is blank
         // This replaces it with our method instead 
         flyout.onCancel = lang.hitch( this, this.onMouseLeave );
       } else {
       
     
       
         if(this.debug){console.log( "[ModelControl::onHover] Unable to locate flyout with ending in " + key + ", recovering as normal hover" );} 
				 // This could be normal enable a mouseleave event on the target node to make this work like a normal hover
				 if( typeof( this.eventReg[ key + "_hover"] ) === "undefined" ){
					   this.eventReg[ key + "_hover" ] = on( evt.currentTarget, "mouseout", lang.hitch( this, this.onHoverOver ) ); 
				 }
       }
       
       Event.stop( evt );

     },
      
     
      
     /* onKeyPress
      * 
      * Handles the keypress on a DOM node that we're watching. Pressing space bar or enter
      * will cause the onHover event to call, show the flyout for the node. 
      *
      */
     onKeyPress: function( evt ) {
      var charOrCode = evt.charCode || evt.keyCode;

      // Look for enter or space and trigger the mouse hover event
      if( charOrCode === keys.ENTER || charOrCode === keys.SPACE ){
         this.onHover( evt );
         Event.stop( evt );
      }
      
     },
		 
    /**
     * onMouseLeave
     *
     * Handle the mouse leave event generated by the DialogTooltips.
     * This will close the tooltip and remove the active theme from the 
     * model thus returning it to the base state.  
     *
     * 
     * Input:
     *  evt - Dojo normalized DOM event.
     *
     * Outputs:
     *  None.
     *
     * Side Effects:
     * None.
     *
     */   
     onMouseLeave: function( evt ) {
       var model = $(".model")[0] || null, flyout, fObj;
       if(this.debug){console.log( "[ModelControl::onMouseLeave] in. " );}
       if( model === null ){
         if(this.debug){console.log( "[ModelControl::onMouseLeave] Unable to locate the model, bail." );}
         return;
       }
       
       // Remove the new highlite to the model
       if( this.activeKey !== null ){
         domClass.remove( model, this.activeKey );
         fObj = this.flyouts[ "flyout-" + this.activeKey ] || null;
         
		 
         if( fObj === null ){
           if(this.debug){console.log( "[ModelControl::onMouseLeave] Unable to locate the flyout in the object trying the registry. " );}
           flyout = registry.byId( "flyout-" + this.activeKey ) || null;
         } else {
           flyout = fObj.flyout;
         }
         
         if( flyout !== null ){
           // For any stack containers, reset them to the first child
           if( flyout.content.toString().indexOf( "StackContainer" ) !== -1 ){
              flyout.content.selectChild( flyout.content.getChildren()[0] );
           }
           popup.close( flyout );
         }
         // Now give the base node that started this all focus 
         $( "."+ this.keyPrefix + this.activeKey )[0].focus();
         this.activeKey = null;
       } else {
         if(this.debug) {console.log( "[ModelControl::onMouseLeave] Huh? No current key, who's calling?" ); } 
       }
     },
     
		/**
     * onHoverOver
     *
     * Handle the mouse leave event generated moving off the model node
		 * when the model does not have a flyout.  
     * This will remove the active theme from the 
     * model thus returning it to the base state.  
     *
     * 
     * Input:
     *  evt - Dojo normalized DOM event.
     *
     * Outputs:
     *  None.
     *
     * Side Effects:
     * None.
     *
     */   

		 onHoverOver: function( evt ) {
		   var model = $(".model")[0] || null, flyout, fObj;
       
       if( model === null ){
         if(this.debug){ console.log( "[ModelControl::onHoverOver] Unable to locate the model, bail." );}
         return;
       }
			 
			 // Remove the class name and reset the key
       //domClass.remove( model, this.activeKey );
			// this.activeKey = null;
			 
		 // Remove the new highlite to the model
       if( this.activeKey !== null ){
       	fObj = this.flyouts[ "flyout-" + this.activeKey ] || null;
       	if(fObj === null){
         domClass.remove( model, this.activeKey );
        }
         //fObj = this.flyouts[ "flyout-" + this.activeKey ] || null;
         
		 
         if( fObj === null ){
           if(this.debug){console.log( "[ModelControl::onMouseLeave] Unable to locate the flyout in the object trying the registry. " );}
           flyout = registry.byId( "flyout-" + this.activeKey ) || null;
         } else {
           flyout = fObj.flyout;
         }
         
         if( flyout !== null ){
           // For any stack containers, reset them to the first child
           if( flyout.content.toString().indexOf( "StackContainer" ) !== -1 ){
              flyout.content.selectChild( flyout.content.getChildren()[0] );
           }
           
           
           //popup.close( flyout );
         }
         // Now give the base node that started this all focus 
         //$( "."+ this.keyPrefix + this.activeKey )[0].focus();
         //this.activeKey = null;
       } else {
         if(this.debug){console.log( "[ModelControl::onMouseLeave] Huh? No current key, who's calling?" ); }
       }	 
			 
			 
			 Event.stop( evt );
			 
		 },
		 
     /**
     * addFlyout
     *
     * This method allows control to keep track of the flyouts
     * so they get destroied during shutdown.
     *
     * 
     * Input:
     *  flyout - Dijit to track.
		 *  orient - Array of positions to try when opening the flyout
     *
     * Outputs:
     *  None.
     *
     * Side Effects:
     *   None.
     *
     */ 
     addFlyout: function( flyout, orient ){
       obj = { flyout: flyout, orient: orient || null };
       this.flyouts[ flyout.id ] = obj;
     },
     
    /**
     * closeActiveFlyout
     *
     * Closes the active known flyout, if there is not a current
     * flyout, it ignores the request.
     *
     * 
     * Input:
     *  None.
     *
     * Outputs:
     *  None.
     *
     * Side Effects:
     * Flyout changes and class style changes.  There is no longer a
     * active state in the model.
     *
     */ 
     closeActiveFlyout: function() {
       console.log( "[ModelControl::closeActiveFlyout] Closing the active key " + this.activeKey );
        if( this.activeKey !== null ) {
          this.onMouseLeave( null );
        }
     },
     
    /**
     * printModel
     *
     * Convert all the flyouts on the page visible elements on the page
		 * in prepreation to print. This will also call up the browser's print
		 * dialog. 
     * 
     * Input:
     *  None.
     *
     * Outputs:
     *  None.
     *
     * Side Effects:
     *   Body contains all the text in the flyouts
     *
     */ 
     printModel: function( ){
       var page = $( ".print" )[0] || null, i, key, tabs, diag, content, btn, bar, children;
     
       if( page !== null ){
         // Already available, so do nothing?
         domConst.destroy( page );     
       }
       page = domConst.create( "div", { className: "print-area" }, $( "body" )[0] );
			 children = $( ".model  *" );

       for( i=0; i < children.length; i+=1 ){
         // Does the node have a key, thus something we want to print
         if( domAttr.has( children[i], "data-key" ) ){
           key = domAttr.get( children[i], "data-key" );
           
            flyout = this._getFlyout( key );
       
      		if( flyout !== null ){
      		//in IMXML v2 all flyouts are activated from a '+' icon, which is within an image, task or role. 	
           this._createPrintHdr( children[i].parentNode.parentNode, page );       
           this._flyoutToDom( key, page );
           }
         } else if( domClass.contains( children[i], "modelTabs" ) ){
					 // This section contains model tabs, lets walk that and see what we find.
					 // Assumption there is only one tab container that is a child element of this div
					 tabs = registry.byNode( $( "> .dijitTabContainer", children[i] )[0] ).getChildren();
					 this._printTabs( tabs, page );
				 } else if( domClass.contains( children[i], "code-section") ){
				   this._printSection( children[i], page );
				 } else if( domAttr.get( children[i], "role" ) == "region" ){
				    this._printRegion( children[i], page, false );
				 }
			 
       }// for(...)
			 
			 // Align the model wrapper
			 domClass.add( $( ".wrapper" )[0], "print-wrapper" );
       
			 // Create a message dialog box to tell the user to enable the printing of background images.
			 // While the two dialog box print process is not ideal, there are just too many question about
			 // why the model is not printing. 
			 content = domConst.create( "div" );
			 domConst.create( "div", { className: "dijitDialogPaneContentArea", 
			                           innerHTML: "This model leverages background images, which most browser do not print by default.  Consult the browser documentation to enable printing of background images." }, content );
			 bar = domConst.create( "div", { className: "dijitDialogPaneActionBar" }, content );
			 
			 // The button action will delay the call to the browser print dialog.  This allows the browser to render before printing
			 // so the dialog box does not appear as part of the content to print. 
			 btn = new Button( { label: "Print", 
			                     onClick: function(){ diag.hide(); window.setTimeout( function(){ window.print(); } , 500); } } );
			 domConst.place( btn.domNode, bar );
																 
			 
			 // Create and show the dialog box.
			 diag = new Dialog( { id: "printDialog",
			                      content: content,
			                      title: "Print Information",
														style: "width: 350px" } );
			 diag.show();
     },
     
		/**
     * _getKey
     *
     * Locate the key in from the target of the event object
     * 
     * Input:
     *  evt - Event Object
     *
     * Outputs:
     *  The dijit or null if not found.
     *
     * Side Effects:
     *  None
     *
     */ 
		 _getKey: function( evt ){
		   return domAttr.get(evt.currentTarget, "data-key" ) || domAttr.get( evt.target, "data-key" ) || null;
		 },
		 
		/**
     * _getFlyout
     *
     * Locate the associated flyout dijit based on the given key.
     * 
     * Input:
     *  key - key to locate the flyout which is flyout-<key>.
     *
     * Outputs:
     *  The dijit or null if not found.
     *
     * Side Effects:
     *  None
     *
     */ 
     _getFlyout: function( key ) {
       var fObj = this.flyouts[ "flyout-" + key ] || null, flyout = null;
       
       if( fObj === null ){
         if(this.debug){console.log( "[ModelControl::onHover] Flyout is not found, looking at registry. " );}
         flyout = registry.byId( "flyout-" + key ) || null;
       }else {
         flyout = fObj.flyout;
       }
       
       return flyout;
     },
     
		/**
     * _flyoutToDom
     *
     * Based on the give key, locate a flyout and then attempt to 
		 * copy the content into the given DOM element.
		 * This method will trys 3 ways to convert the flyout content:
		 * 1. Call the print method of the flyout content
		 * 2. Copy the text of flyout content if a string
		 * 3. Clone the domNode of a dijit 
     *
     * 
     * Input:
     *  key - key to locate the flyout which is flyout-<key>
		 *  page - DOM element to append the conversion
     *
     * Outputs:
     *  None.
     *
     * Side Effects:
     *  None
     *
     */ 
     _flyoutToDom: function( key, page ){
        // Figure out how to render the node
       flyout = this._getFlyout( key );
       
       if( flyout !== null ){
           if( flyout.content.toString().indexOf( "StackContainer" ) !== -1 ){
           this._printStack( page, flyout.content );
         } else if( typeof( flyout.content.print ) === "function" ){
           // Must be something we created, so call the print method
           flyout.content.print( page );
         } else {
           // Must be text or something else so just clone the content
           if( typeof( flyout.content ) === "string" ){
             domConst.create( "div", { innerHTML: flyout.content }, page );
           } else if( !typeof( flyout.content.domNode) === "undefined"){
             domConst.place( lang.clone( flyout.content.domNode ), page );
           } else {
             // Don't understand what this is
             if(this.debug){ console.log( "[ModelControl::onPrint] Encounter something that is not text or a Dijit " + flyout.content.toString() );}
           }
         }
       }
     },
     
		/**
     * _printStack
     *
     * Given a dijit.layout.StackContainer convert each page
		 * in the stack in an indented section.
     *
     * 
     * Input:
		 *  page - DOM element to append the conversion
		 *  stack - StackContainer to print
     *
     * Outputs:
     *  None.
     *
     * Side Effects:
     *  None
     *
     */ 
		 _printStack: function( page, stack ){
		  var panes = stack.getChildren(), i, indent;
			
			indent = domConst.create( "div", {className: "indent"}, page );
			
			for( i = 0; i < panes.length; i += 1 ){
				 if( typeof( panes[i].print ) === "function" ){
           // Must be something we created, so call the print method
           panes[i].print( indent );
         } else if( typeof( panes[i].content ) === "string" ){
					 // Appears to be only text, create a div (since IE 8 throw execption on p)
           domConst.create( "div", { innerHTML: flyout.content }, indent );
         } else if( !typeof( panes[i].domNode) === "undefined"){
					 // Must be a dijit so just clone it's content and put it on the page
           domConst.place( lang.clone( panes[i].domNode ), indent );
         } else {
             // Don't understand what this is
             if(this.debug){console.log( "[ModelControl::_printStack] Encounter something that is not text or a Dijit " + flyout.content.toString() );}
         }			
			}
			
				 
		},
		
		/**
     * _printTabs
     *
     * Given the children elements of a dijit.layout.TabContainer
		 * convert each page in the stack in an indented section. The 
		 * tab's title is used to create a heading for each indented 
		 * section. 
     * 
     * Input:
		 *  tabs - Array of tabs in the tab container
		 *  page - DOM element to append the conversion
     *
     * Outputs:
     *  None.
     *
     * Side Effects:
     *  None
     *
     */ 
		_printTabs: function( tabs, page ){
			var i;
				
			for( i=0; i < tabs.length; i+=1 ){
				// Create a header based on the tab heading
				domConst.create( "h1", { innerHTML: tabs[i].title }, page );
				this._printSection( tabs[i].domNode, page );
				
			} // for(i)
		},

		/**
     * _printSection
     *
     * Print a section in a tab or other call out box.  
     * 
     * Input:
		 *  tabs - Array of tabs in the tab container
		 *  page - DOM element to append the conversion
     *
     * Outputs:
     *  None.
     *
     * Side Effects:
     *  None
     *
     */ 
    _printSection: function( domNode, page ){
			var nodes, j, indent, hdr;
			
			
			// Extract all the node that we can evaluate and create the indent section
			nodes = $( "div[data-key]", domNode );
			indent = domConst.create( "div", { className: "indent" }, page );
			
			// Convert the flyouts into the dom for each of these keys
			for( j=0; j < nodes.length; j+=1 ){
				this._createPrintHdr( nodes[j], indent );
				this._flyoutToDom( domAttr.get( nodes[j], "data-key" ), indent );
			}

		},
		
		
		/**
     * _printSection
     *
     * Print an identified region in the code using recurssion.
		 * This method is similar to _printSection, but takes advantage
		 * of the hierarchy to display the children in order.  After
		 * processing direct siblings of the given domNode, it look for 
		 * other regions and prints those.
     * 
     * Input:
		 *  tabs - Array of tabs in the tab container
		 *  page - DOM element to append the conversion
		 *  indent - Boolean
     *
     * Outputs:
     *  None.
     *
     * Side Effects:
     *  None
     *
     */ 
		_printRegion: function( domNode, page, indent ) {
			var nodes, j, hdr, block = page;


	  	// Output any internal header statements, scope only to direct children
			hdr = $( "> h1", domNode )[0] || $( "> h2", domNode )[0] || domAttr.get( domNode, "aria-label") || null;
			
			
			if( hdr != null && typeof(hdr) === "string" ){
			  domConst.create( "h1", { innerHTML: hdr }, page );			
			} else {
			  domConst.create( "h1", { innerHTML: hdr.innerHTML }, page );			
			}

			// Extract all the node that we can evaluate and create the indent section
			nodes = $( "> div[data-key]", domNode );
			if( indent ){
				block = domConst.create( "div", { className: "indent" }, page );
			}

			// Convert the flyouts into the dom for each of these keys
			for( j=0; j < nodes.length; j+=1 ){
				this._createPrintHdr( nodes[j], block );
				this._flyoutToDom( domAttr.get( nodes[j], "data-key" ), block );
			}

      // Do we have other regions to deal with
			nodes = $( "> div[role=region]", domNode ) || null;
			
			// Use recrusion to display those levels, indenting each time.
			if( nodes !== null ){
				for( j=0; j < nodes.length; j+=1 ){
				   this._printRegion( nodes[j], block, true );
				}
			}
			
			return;
		},
		
		/**
     * _createPrintHdr
     *
     * Given a node in the model determine an appopiate header.
		 * If the node represents a ball in the diagram, a special 
		 * ball header is generated with a smaller ball.  If the node
		 * represents anything else, the background image is placed on the
		 * left of the header and text (minus any HTML) as the header text.
     * 
     * Input:
		 *  node - DOM element to convert into a header
		 *  page - DOM element to append the conversion
     *
     * Outputs:
     *  None.
     *
     * Side Effects:
     *  The image may fliker on the screen during construction since
		 *  it is necessary to place the image in the DOM to determine
		 *  the size. 
     *
     */ 
		 _createPrintHdr: function( node, page ){
			  var hdr, url, img, box, idx = node.className.indexOf( 'ball' ), list, name = null;
				 
		   // Convert the node into a header
			 hdr = domConst.create( "h1", null, page );
			 
			 // Are we dealing with a ball?
			 if( false ){
				 // Copy the ball class name over to the h1 and let the CSS role with it
				 list = node.className.split(/\s+/);
				 idx = 0;
				 
				 // Locate the ball class name, assume there is only one.
				 while( name === null && list.length > idx ){
					 if( list[idx].indexOf( "ball" ) >= 0 ){
					   name = list[idx];
					 }else{
						 idx += 1;
					 }
				 }
				 
				 if( name !== null ){
				   domClass.add( hdr, name );
				 } else {
					 if(this.debug){console.log( "[_createPrintHdr] Indicated there was a ball in the class name but none was found after split." );}
				 }
				 
			 } else if( node.textContent || node.innerText) {
				 // Determine the size of the background image
				 url = domStyle.get( node, "backgroundImage" );
				 						 console.log( "Preping print image url:" + url );
				 url = url.substring( 5, url.length - 2 );
				 img = domConst.create( "img", { src: url }, hdr );
				 box = domGeo.getContentBox( img );
				 
				 // Adjust the header style to use the same background image
				 domStyle.set( hdr, { lineHeight: box.h + "px",
															paddingLeft: ( box.w + 5 ) + "px",
															backgroundImage: domStyle.get( node, "backgroundImage" ) } );
				 domConst.destroy( img );
			 }
														
			 // Add the text to the node, for nodes with no text use a non-blank space
			 txt = win.doc.createTextNode( node.textContent || node.innerText || "\u00A0" );
			 domConst.place( txt, hdr );
			 
		 }
  } ); // declare
  

  
}); //