/*
Licensed Materials - Property of IBM

 Copyright IBM Corp. 2012 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or
disclosure restricted by GSA ADP Schedule Contract with
IBM Corp.
*/
define( [ "dojo/_base/declare",
          "dijit/_WidgetBase",
          "dojo/dom-construct",
					"dojo/dom-geometry",
					"dojo/dom-style",
					"dojo/dom-class",
					"dojo/dom-attr",
					"dojo/query",
					"dojo/on",
					"dojo/_base/lang",
					"dijit/registry",
					"dojo/_base/event",
					"dojo/keys",
					"dijit/a11y" ],
					function( declare, _WidgetBase, domConst, domGeo, domStyle, domClass, domAttr, $, on, lang, registry, Event, keys, a11y ){
						
	 return declare( "StackOptionList", _WidgetBase, {
		 
		 /*  StackOptionList, 
		     
				 JSON Format:
				 { opening: "text", 
				   items: [ {title: "title", desc: "desc", key: "otherPanelName"} ] } */
		 data: null,
		 
		 // Handle back to the stack container that this object will control
		 stackContainer: null,
		 
		  /* itemClick 
		  *
			* Create the item list for the event object. 
			*
			* Inputs:
			*  None.
			*
			* Output:
			*  None.
			*
			* Side effect:
			*   Stops the default event action 
			*
			*/
		 postCreate: function( ) {
			 var i, clickFunc = lang.hitch( this, this.itemClick ), keyFunc = lang.hitch( this, this.onKeypress );

			 if( this.data === null || typeof(this.data.options) === undefined ){
				 console.error( "[StackOptionList:postCreate] No option list provided" );
				 return;
			 }
			 
				if( typeof(this.data.opening ) !== "undefined"){
				  	domConst.create( "div", {className: "desc", innerHTML: this.data.opening }, this.domNode );
				}
				
				for( i = 0; i < this.data.items.length; ++i ){
					var itemNode = domConst.create( "div", { className: "listItem", tabIndex: "0", role: "button" }, this.domNode );
					domAttr.set( itemNode, "data-stackIdx", this.data.items[i].key );
					on( itemNode, "click", clickFunc );
					on( itemNode, "keypress", keyFunc );
					
					domConst.create( "h2", { innerHTML: this.data.items[i].title }, itemNode );
					domConst.create( "div", { innerHTML: this.data.items[i].desc }, itemNode );
					
				}
			 			 
		 },
		 
		 /* itemClick 
		  *
			* Public function to control the switching of the page based on a normalized dom event. 
			* This mehtod assumes that either the current target or target node will have an attribute
			* of data-stackIdx.  That attribute is the id of the next page in the stack that we want to 
			* display.  However, when that is not found it will show an error message on the console.
			*
			* Inputs:
			* evt - Normailzed DOM Event object from Dojo
			*
			* Output:
			*  None.
			*
			* Side effect:
			*   Stops the default event action 
			*
			*/
		 itemClick: function( evt ){
		  var dest = domAttr.get(evt.currentTarget, "data-stackIdx" ) || domAttr.get( evt.target, "data-stackIdx" );
			console.log("Moving stack to destination " + dest);
			dest = registry.byId( dest );
			if( typeof( dest ) !== "undefined" && this.stackContainer.getIndexOfChild( dest ) !== -1 ){
			  this.stackContainer.selectChild( dest );
				
				// Set focus on the first thing in teh tabbing order we can find on the page 
				a11y.getFirstInTabbingOrder( dest.domNode ).focus();
				
			} else {
				console.error( "Unable to locate the destination widget to change pages, either it does not exist or did not get loaded into the stack." );
			}
			Event.stop( evt );
		 },
		 
		 onKeypress: function( evt ){
		  var charOrCode = evt.charCode || evt.keyCode;
			
		  // Look for enter or space and trigger the mouse hover event
			if( charOrCode === keys.ENTER || charOrCode === keys.SPACE ){
			   this.itemClick( evt );
				 Event.stop( evt );
			}
			
			// ignore it other wise and let the default action happen
		 },
		 
		 print: function( node ){
		   console.log( "Printing..." + this.id );
		 }
		 
		 
	 } ); // declare
  
}); //
		