/*
* IBM Confidential
* OCO Source Materials
* (C) Copyright IBM Corp. 2012
* The source code for this program is not published
* or otherwise divested of its trade secrets, irrespective
* of what has been deposited with the U.S. Copyright Office.
*/ 
define( [ "dojo/_base/declare",
          "dijit/_WidgetBase",
          "dojo/dom-construct",
					"dojo/dom-geometry",
					"dojo/dom-style",
					"dojo/dom-class",
					"dojo/dom-attr",
					"dojo/query",
					"dojo/on",
					"dojo/_base/lang" ],
					function( declare, _WidgetBase, domConst, domGeo, domStyle, domClass, domAttr, $, on, lang ){
						
	 return declare( "StepCard", _WidgetBase, {
		
		/* cardData - JSON object describing the content of the card
		   { opening: "[ text | html ]", 
			   steps: [ { label: "[ text | html] ", imgSrc: "url", imgAlt: "string" }, 
				          ... ],
				 closing: "[ text | html ]",
				 action: { label: "text", url: "url", target: "target", attr: {name: "name", value: "value"}, click: function }
			 }
			 
			 opening - An optional opening text to display to the user before presenting the steps.
			           This can be plain text or HTML code. Use the opening when the card is only 
								 displaying text.
			 steps - List of steps the user will perform to complete the task.
			         This is an JS Object with the following attributes:
							 
							 label - Text or HTML string shown to the user.
							 imgSrc - Optional image to display when the user selects the step.
							 imgAlt - Alt text to display when the user hovers over the image.
							 
			 closing - An optional closing text display to the user after presenting the steps.
			 action - An optional link back to where ever for information. This will appear as 
			            as a hyperlink at the bottom of the card.  This is an JS Object
			            with the following attributes:
									
								 label - Text to display to the user.
								 url - URL to launch in the target browser window
								 target - Window target by default this is _blank to open a new browser windwow 
								 attribute - An attribute to add to the action element.    
     	*/
			cardData: null,
			
			/* CSS Class name for the layout. This allows adjustment of the column widths ratios, and other attributes.
			   For example:
				 
				 .twoCol {
						position: relative;
						padding: 10px;
					}
					
					.twoCol .col1{
						width: 51%;
						padding-right: 2%;
					}
					
					.twoCol .col2{
						position: absolute;
						top: 0;
						left: 51%;
						right:0;
						bottom: 0;
						overflow: auto;
					}
			*/
			layoutRule: "twoCol",
			
			// Image DOM element 
			img: null,
				
			// Arrow color used to highlight when step is active 
			arrowColor: "blue",
			
			// List of steps		
			stepList: [],
			
			// Current Step 
			stepCurrent: 0,
			
			// Index of the first step with images, used to set the default when tooltip opens
			firstStepWithImg: -1,
			
			// Indicates there are steps with images 
		  hasStepsImages: false,
				
		  // Show the image only when printing
			imgPrintOnly: false,
			
			
	   /* postCreate
      * 
      * Converts the cardData into HTML and appends that to the 
			* base domNode of the dijit.  This method is forgiving, so if
			* your missing a opening, closing, steps, step images, ect, it
			* just adjusts the layout.  This method can produce a one column
			* or two column layout.  The one column layout is for those steps
			* that either have no images or imgPrintOnly is set.  The two 
			* column layout is generated for those steps with images that 
			* get displayed on click. 
      *
      * Input:
      *  None.
      * 
      * Output:
      * None.
      *
      * Side Effects:
      *   None.
      *
      */    
			postCreate: function(){
				this.inherited(arguments);
				var dat = this.cardData, i;
				
				// Add our classname to the base class, but it appear we need to add it to the domNode as well
				// since this class does not support the typically state changes.
			  this.baseClass = this.baseClass + " stepHoverCard";
        domClass.add( this.domNode, "stepHoverCard" );
				
				// Determine if we have steps and if those steps have images
				// This will determine the layout and how we proceed with construction.
				var hasSteps = !(typeof(dat.steps) === "undefined");
				var steps = null;
				if( hasSteps ){
					steps = dat.steps;

					// Determine if the steps have images 
					if( !this.imgPrintOnly ){
						for( i =0; i < steps.length && !this.hasStepsImages; ++i ){
								if( !(typeof (steps[i].imgSrc) === "undefined") ) {
									this.hasStepsImages = true;
								}
						} // For(...)
					}// if(...)
				}
				
				// Create the text column that will contain all the text
				var textCol = domConst.create( "div", { className: "col1" } );
				
				// Add the opening text if it exists 
				if( typeof(dat.opening ) !== "undefined"){
				  	domConst.create( "div", { innerHTML: dat.opening, className: "opening" }, textCol );
				}
				

				// Work through the step creation, if available.
				if( hasSteps ){
					// Create the order list element and hitch the updateImage function
				  var stepList = domConst.create( "ol", {className: "steps" }, textCol );
					var updateFunc = lang.hitch( this, this.updateImage ); 
					
					for( i = 0; i < steps.length; ++i ){
            var item = domConst.create( "li", null, stepList );
						this.stepList.push( item );
						
						// Determine if this step has an image or not
						if( (typeof( steps[i].imgSrc ) === "undefined" ) || this.imgPrintOnly ){
							// No image so just update the text of the list item 
							item.innerHTML = steps[i].label;
						} else {
							// There is an image so create add all the elements to the list item, an arrow and anchor 
							domConst.create( "div", { className: "arrow",
 						 																			 	id: this.id + "_arrow_" + i }, item );
							
							// Track the first step with images so we can make that the default later
							if( this.firstStepWithImg === -1 ){
								 this.firstStepWithImg = i;
							}
							var itemTouch = domConst.create( "a", { innerHTML: steps[i].label,
							 																				 href: "#", id: this.id + "_step_" + i,
																											 onclick: updateFunc }, item );
							domAttr.set( itemTouch, "stepIdx", i );
						}
					} // for(steps)
				} //if( hasSteps )
			
				// Add the action, if any
				// Add the closing text, if any
				if( !(typeof(dat.closing ) === "undefined" ) ){
				  	domConst.create( "div", {className: "closing", innerHTML: dat.closing }, textCol );
				}
				
				// Add the action text, if any
				if( !(typeof(dat.action ) === "undefined" ) ){
				  	var a = domConst.create( "a", {target: dat.action.target || "",
						                       innerHTML: dat.action.label,
																	 title: dat.action.label,
																	 href: dat.action.url || "#",
																	 className: "stepAction" }, textCol );
					  if( typeof( dat.action.onClick ) === "function" ){
							on( a, "click", dat.action.onClick );
						} 
						if( !(typeof( dat.action.attr ) === "undefined" ) ){
							domAttr.set( a, dat.action.attr.name, dat.action.attr.value ); 
						}
				}

				// Now determine if we need to create the image column.
				if( this.hasStepsImages ){
					// create the image pane with a two column layout
					var wrapper = domConst.create( "div", { className: this.layoutRule } );
					domConst.place( textCol, wrapper );
					var imgCol = domConst.create( "div", { className: "col2 imgCol" }, wrapper );
					
					// Add the image 
					this.img = domConst.create( "img", { src: this._blankGif, alt: "" }, imgCol );
					
					domConst.place( wrapper, this.domNode );
					
					//this.startup();
				} else {
					domClass.remove( textCol, "col1" );
					domConst.place( textCol, this.domNode ); 
				}

			}, // postCreate
			
			
		 /* startup
      * 
      * Shows the image of the first available step.  If you don't 
			* call this method, the image column will be blank. 
      *
      * Input:
      *  None.
      * 
      * Output:
      * None.
      *
      * Side Effects:
      *   None.
      *
      */    
			startup: function(  ){
				this.showImage( this.firstStepWithImg );
			},
			
		 /* updateImage
      * 
      * Handles the click to change which image is currently 
			* displayed in the flyout. 
      *
      * Input:
      *  evt - Normalized DOM Event object.
      * 
      * Output:
      *  None.
      *
      * Side Effects:
      *   This stops the event processing.
      *
      */    
			updateImage: function( evt ){
				console.log( "[updateImage] Adjusting steps." );
				this.showImage( parseInt( domAttr.get( evt.currentTarget, "stepIdx" ) ) );
				
				// Stop the event, so we don't go anywhere
				evt.stopPropagation(); 
				evt.preventDefault();
			},
			
		 /* showImage
      * 
      * Update the image column in the flyout with the image
			* related to the given index.  If the index is bogus
			* it just ignores the request.
      *
      * Input:
      *  stepIdx - A number representing the step image to display
      * 
      * Output:
      *  None.
      *
      * Side Effects:
      *   This stops the event processing.
      *
      */    
			showImage: function( stepIdx ){
				var arrow;
				
				// check if we are in range
				if( stepIdx < 0 || stepIdx > this.cardData.steps.length ){
					if( this.hasStepImages ){
					  console.warn( "[StepHoverCard::showImage] Invalid step index " + stepIdx );
					}
				  return;
				}
				
				// Remove any previous selection
				if( stepIdx != this.stepCurrent ){
				   arrow = $( "#" + this.id + "_arrow_" + this.stepCurrent, this.domNode )[0] || null;
					 
					 if( arrow !== null ){
						 domClass.remove( arrow, this.arrowColor );
					 }
				}
				
				// Indicate the new selection 
				arrow = $( "#" + this.id + "_arrow_" + stepIdx, this.domNode )[0] || null;
					 
			  if( arrow !== null ){
						 domClass.add( arrow, this.arrowColor );
						 this.stepCurrent = stepIdx;
				} else {
					console.error( "[StepHoverCard::showImage] Unable to locate the arrow for step " + stepIdx );
				}
				
				// Show the image for the step
				domAttr.set( this.img, { src: this.cardData.steps[stepIdx].imgSrc, alt: this.cardData.steps[stepIdx].imgAlt } );

			},

		 /* print
      * 
      * Generates a special output of the cardData that is inactive
			* but shows all the images and text in the steps. The new 
			* DOM nodes are added to the given node. Actions are currnetly
			* ignored and not added to the print content.
      *
      * Input:
      *  node - DOM node to append the output of the conversion.
      * 
      * Output:
      *  None.
      *
      * Side Effects:
      *   None.
      *
      */    
			print: function( node ){
				var dat = this.cardData, i, steplist, itm, img, card;
				
				card = domConst.create( "div", {className: "stepHoverCard"}, node );
				
				
				// Add the opening text if it exists 
				if( typeof(dat.opening ) !== "undefined"){
				  	domConst.create( "div", { innerHTML: dat.opening, className: "opening" }, card );
				}
				
				if( typeof(dat.steps) !== "undefined" ){
				  stepList = domConst.create( "ol", {className: "steps" }, card );
					
				  for( i=0; i< dat.steps.length; i+=1 ){
						itm = domConst.create( "li", { innerHTML: dat.steps[i].label }, stepList );
						
						// Add the image below the step text
						if( typeof( dat.steps[i].imgSrc ) !== "undefined" ){
						  domConst.create( "br", null, itm );
							domConst.create( "img", { src: dat.steps[i].imgSrc, alt: dat.steps[i].imgAlt || "" }, itm ); 
						} 				
					}//for()
				} //if( has steps ) 
				
				// Add the closing, if it exists 
				if( typeof(dat.closing ) !== "undefined"){
				  	domConst.create( "div", { innerHTML: dat.closing, className: "closing" }, card );
				}
				
				// Actions?????
			}
	} ); // declare
  
}); //